//
//  AppDelegate.m
//

#import "AppDelegate.h"

@interface AppDelegate ()

@property (weak) IBOutlet NSWindow *window;
@property (weak) IBOutlet NSTextField *calibratedTextField;
@property (weak) IBOutlet NSTextField *sRGBTextField;

@property (weak) IBOutlet NSTextField *backgroundTextField;

@end

@implementation AppDelegate

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
	// NOTE: You can be notified if a window's color space changes. This can happen if the user goes to System Preferences
	// and changes the Display's color profile or if a window moves from one screen to another.
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(windowDidChangeBackingProperties:) name:NSWindowDidChangeBackingPropertiesNotification object:nil];
	
	
	// NOTE: When you create "calibrated" color, you're making a color with the Generic RGB color space.
	self.calibratedTextField.backgroundColor = [NSColor colorWithCalibratedRed:(123.0/255.0) green:(45.0/255.0) blue:(67.0/255.0) alpha:1.0];
	self.calibratedTextField.stringValue = @"123, 45, 67";


	// NOTE: Starting with OS X 10.9, sRGB color can also be created with:
	//
	//	[NSColor colorWithRed:(123.0/255.0) green:(45.0/255.0) blue:(67.0/255.0) alpha:1.0];
	//
	// For cross-platform projects this can be helpful: the method signature is identical to the one used on iOS.
	self.sRGBTextField.backgroundColor = [NSColor colorWithSRGBRed:(123.0/255.0) green:(45.0/255.0) blue:(67.0/255.0) alpha:1.0];
	self.sRGBTextField.stringValue = @"123, 45, 67";
}

- (void)setBackgroundColorUsingColorSpace:(NSColorSpace *)colorSpace
{
	CGFloat components[] = { (102.0/255.0), (0.0/255.0), (204.0/255.0), 1.0 };
	if (colorSpace) {
		NSLog(@"%s updating backgroundTextField using %@", __PRETTY_FUNCTION__, colorSpace);
		NSColor *color = [NSColor colorWithColorSpace:colorSpace components:components count:4];
		self.backgroundTextField.backgroundColor = color;
		self.backgroundTextField.stringValue = colorSpace.localizedName;
	}
}

// NOTE: If you need more direct control, you can create colors using the RGB components and a color space. Some examples
// are shown below:

- (IBAction)setGenericRGBColorSpace:(id)sender
{
	// 1) NSColorSpace has several class methods for common color spaces. Generic RGB is the default color space.
	NSColorSpace *colorSpace = [NSColorSpace genericRGBColorSpace];
	[self setBackgroundColorUsingColorSpace:colorSpace];
}

- (IBAction)setSRGBColorSpace:(id)sender
{
	// 2) This produces the same color as the -colorWithSRGBRed:green:blue:alpha method
	NSColorSpace *colorSpace = [NSColorSpace sRGBColorSpace];
	[self setBackgroundColorUsingColorSpace:colorSpace];
}


- (IBAction)setScreenColorSpace:(id)sender
{
	// 3) Each window has a screen where it currently resides. Each screen has a color space which can be used to create
	// a color that's in the display's native values (in the Digital Color Meter.) Use this instead of deviceRGBColorSpace.
	NSColorSpace *colorSpace = self.window.screen.colorSpace;
	[self setBackgroundColorUsingColorSpace:colorSpace];
}

- (IBAction)setDisplayP3ColorSpace:(id)sender
{
	// 4) Color spaces can be created using Core Graphics functions. Since there is currently no class method to get the
	// expanded color gamut of Display P3 (on the new iMacs), this is a convenient way to create the color space.
	CGColorSpaceRef colorSpaceRef = CGColorSpaceCreateWithName(kCGColorSpaceDisplayP3);
	NSColorSpace *colorSpace = [[NSColorSpace alloc] initWithCGColorSpace:colorSpaceRef];
	CGColorSpaceRelease(colorSpaceRef);
	[self setBackgroundColorUsingColorSpace:colorSpace];
}

- (IBAction)setMonitorRGBColorSpace:(id)sender
{
	// 5) If you have specific color spaces that you need to use in your app, the ICC profiles can be loaded from your bundle.
	// This example uses the same profile as the "Monitor RGB" image (ColorTest-MonitorRGB in the asset catalog). This
	// allows you to match the image and background color exactly.
	NSURL *iccProfileURL = [[NSBundle mainBundle] URLForResource:@"Monitor" withExtension:@".icc"];
	NSData *iccProfileData = [NSData dataWithContentsOfURL:iccProfileURL];
	NSColorSpace *colorSpace = [[NSColorSpace alloc] initWithICCProfileData:iccProfileData];
	[self setBackgroundColorUsingColorSpace:colorSpace];
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)application
{
	return YES;
}

- (void)windowDidChangeBackingProperties:(NSNotification *)notification
{
	NSWindow *window = (NSWindow *)notification.object;
	NSDictionary *userInfo = notification.userInfo;
	NSLog(@"%s window = %@, userInfo = %@", __PRETTY_FUNCTION__, window, userInfo);
 
	CGFloat newBackingScaleFactor = window.backingScaleFactor;
	CGFloat oldBackingScaleFactor = [(NSNumber *)[userInfo objectForKey:NSBackingPropertyOldScaleFactorKey] doubleValue];
	if (newBackingScaleFactor != oldBackingScaleFactor) {
		NSLog(@"%s backing scale factor changed from %.1f to %.1f", __PRETTY_FUNCTION__, oldBackingScaleFactor, newBackingScaleFactor);
	}
 
	NSColorSpace *newColorSpace = window.colorSpace;
	NSColorSpace *oldColorSpace = (NSColorSpace *)[userInfo objectForKey:NSBackingPropertyOldColorSpaceKey];
	if (! [newColorSpace isEqual:oldColorSpace]) {
		NSLog(@"%s the color space changed from '%@' to '%@'", __PRETTY_FUNCTION__, oldColorSpace.localizedName, newColorSpace.localizedName);
	}
	
	if ([self.backgroundTextField.backgroundColor.colorSpace isEqual:oldColorSpace]) {
		NSLog(@"%s updating backgroundTextField using %@", __PRETTY_FUNCTION__, newColorSpace);
		CGFloat components[] = { (102.0/255.0), (0.0/255.0), (204.0/255.0), 1.0 };
		NSColor *color = [NSColor colorWithColorSpace:newColorSpace components:components count:4];
		self.backgroundTextField.backgroundColor = color;
		self.backgroundTextField.stringValue = newColorSpace.localizedName;
	}
}

@end
